/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../tellico-4.2.1/src/mainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Tellico10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto Tellico::MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN7Tellico10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Tellico::MainWindow",
        "slotInit",
        "",
        "slotFileNew",
        "type",
        "slotFileNewByTemplate",
        "collectionTemplate",
        "slotFileOpen",
        "QUrl",
        "url",
        "slotFileOpenRecent",
        "slotFileSave",
        "slotFileSaveAs",
        "slotFileSaveAsTemplate",
        "slotFilePrint",
        "slotFilePrintPreview",
        "slotFileQuit",
        "slotEditCut",
        "slotEditCopy",
        "slotEditPaste",
        "slotEditSelectAll",
        "slotEditDeselect",
        "slotToggleEntryEditor",
        "slotShowConfigDialog",
        "slotHideConfigDialog",
        "slotShowFetchDialog",
        "slotHideFetchDialog",
        "slotStatusMsg",
        "text",
        "slotClearStatus",
        "slotEntryCount",
        "slotHandleConfigChange",
        "slotChangeGrouping",
        "slotFileImport",
        "format",
        "slotFileExport",
        "slotShowFilterDialog",
        "slotHideFilterDialog",
        "slotShowCollectionFieldsDialog",
        "slotHideCollectionFieldsDialog",
        "slotShowStringMacroDialog",
        "slotShowBibtexKeyDialog",
        "slotHideBibtexKeyDialog",
        "slotURLAction",
        "slotEnableOpenedActions",
        "slotEnableModifiedActions",
        "modified",
        "readCollectionOptions",
        "Tellico::Data::CollPtr",
        "coll",
        "saveCollectionOptions",
        "slotQueueFilter",
        "slotCheckFilterQueue",
        "slotUpdateFilter",
        "Tellico::FilterPtr",
        "filter",
        "slotUpdateCollectionToolBar",
        "slotNewEntry",
        "slotEditDialogFinished",
        "slotStringMacroDialogFinished",
        "result",
        "slotUpdateToolbarIcons",
        "slotConvertToBibliography",
        "slotCiteEntry",
        "action",
        "slotShowEntryEditor",
        "slotShowReportDialog",
        "slotHideReportDialog",
        "slotGroupLabelActivated",
        "slotFilterLabelActivated",
        "slotClearFilter",
        "slotRenameCollection",
        "slotImageLocationMismatch",
        "slotImageLocationChanged",
        "slotToggleFullScreen",
        "slotToggleMenuBarVisibility",
        "slotToggleLayoutLock",
        "lock",
        "slotResetLayout",
        "guiFactoryReset",
        "showLog"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotInit'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileNew'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'slotFileNewByTemplate'
        QtMocHelpers::SlotData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'slotFileOpen'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileOpen'
        QtMocHelpers::SlotData<void(const QUrl &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotFileOpenRecent'
        QtMocHelpers::SlotData<void(const QUrl &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotFileSave'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileSaveAs'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileSaveAsTemplate'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFilePrint'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFilePrintPreview'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileQuit'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditCut'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditCopy'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditPaste'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditSelectAll'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEditDeselect'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotToggleEntryEditor'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowConfigDialog'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideConfigDialog'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowFetchDialog'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideFetchDialog'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStatusMsg'
        QtMocHelpers::SlotData<void(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 },
        }}),
        // Slot 'slotClearStatus'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEntryCount'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHandleConfigChange'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChangeGrouping'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileImport'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotFileExport'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'slotShowFilterDialog'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideFilterDialog'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowCollectionFieldsDialog'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideCollectionFieldsDialog'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowStringMacroDialog'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowBibtexKeyDialog'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHideBibtexKeyDialog'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotURLAction'
        QtMocHelpers::SlotData<void(const QUrl &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotEnableOpenedActions'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEnableModifiedActions'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 46 },
        }}),
        // Slot 'slotEnableModifiedActions'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'readCollectionOptions'
        QtMocHelpers::SlotData<void(Tellico::Data::CollPtr)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'saveCollectionOptions'
        QtMocHelpers::SlotData<void(Tellico::Data::CollPtr)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'slotQueueFilter'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCheckFilterQueue'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdateFilter'
        QtMocHelpers::SlotData<void(Tellico::FilterPtr)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'slotUpdateCollectionToolBar'
        QtMocHelpers::SlotData<void(Tellico::Data::CollPtr)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'slotNewEntry'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditDialogFinished'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotStringMacroDialogFinished'
        QtMocHelpers::SlotData<void(int)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 60 },
        }}),
        // Slot 'slotStringMacroDialogFinished'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotUpdateToolbarIcons'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotConvertToBibliography'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCiteEntry'
        QtMocHelpers::SlotData<void(int)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 64 },
        }}),
        // Slot 'slotShowEntryEditor'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowReportDialog'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHideReportDialog'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGroupLabelActivated'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFilterLabelActivated'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotClearFilter'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRenameCollection'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotImageLocationMismatch'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotImageLocationChanged'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotToggleFullScreen'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotToggleMenuBarVisibility'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotToggleLayoutLock'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 77 },
        }}),
        // Slot 'slotResetLayout'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'guiFactoryReset'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showLog'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN7Tellico10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Tellico::MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Tellico10MainWindowE_t>.metaTypes,
    nullptr
} };

void Tellico::MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotInit(); break;
        case 1: _t->slotFileNew((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->slotFileNewByTemplate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->slotFileOpen(); break;
        case 4: _t->slotFileOpen((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 5: _t->slotFileOpenRecent((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 6: _t->slotFileSave(); break;
        case 7: _t->slotFileSaveAs(); break;
        case 8: _t->slotFileSaveAsTemplate(); break;
        case 9: _t->slotFilePrint(); break;
        case 10: _t->slotFilePrintPreview(); break;
        case 11: _t->slotFileQuit(); break;
        case 12: _t->slotEditCut(); break;
        case 13: _t->slotEditCopy(); break;
        case 14: _t->slotEditPaste(); break;
        case 15: _t->slotEditSelectAll(); break;
        case 16: _t->slotEditDeselect(); break;
        case 17: _t->slotToggleEntryEditor(); break;
        case 18: _t->slotShowConfigDialog(); break;
        case 19: _t->slotHideConfigDialog(); break;
        case 20: _t->slotShowFetchDialog(); break;
        case 21: _t->slotHideFetchDialog(); break;
        case 22: _t->slotStatusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->slotClearStatus(); break;
        case 24: _t->slotEntryCount(); break;
        case 25: _t->slotHandleConfigChange(); break;
        case 26: _t->slotChangeGrouping(); break;
        case 27: _t->slotFileImport((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->slotFileExport((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->slotShowFilterDialog(); break;
        case 30: _t->slotHideFilterDialog(); break;
        case 31: _t->slotShowCollectionFieldsDialog(); break;
        case 32: _t->slotHideCollectionFieldsDialog(); break;
        case 33: _t->slotShowStringMacroDialog(); break;
        case 34: _t->slotShowBibtexKeyDialog(); break;
        case 35: _t->slotHideBibtexKeyDialog(); break;
        case 36: _t->slotURLAction((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 37: _t->slotEnableOpenedActions(); break;
        case 38: _t->slotEnableModifiedActions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->slotEnableModifiedActions(); break;
        case 40: _t->readCollectionOptions((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 41: _t->saveCollectionOptions((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 42: _t->slotQueueFilter(); break;
        case 43: _t->slotCheckFilterQueue(); break;
        case 44: _t->slotUpdateFilter((*reinterpret_cast<std::add_pointer_t<Tellico::FilterPtr>>(_a[1]))); break;
        case 45: _t->slotUpdateCollectionToolBar((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 46: _t->slotNewEntry(); break;
        case 47: _t->slotEditDialogFinished(); break;
        case 48: _t->slotStringMacroDialogFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 49: _t->slotStringMacroDialogFinished(); break;
        case 50: _t->slotUpdateToolbarIcons(); break;
        case 51: _t->slotConvertToBibliography(); break;
        case 52: _t->slotCiteEntry((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 53: _t->slotShowEntryEditor(); break;
        case 54: _t->slotShowReportDialog(); break;
        case 55: _t->slotHideReportDialog(); break;
        case 56: _t->slotGroupLabelActivated(); break;
        case 57: _t->slotFilterLabelActivated(); break;
        case 58: _t->slotClearFilter(); break;
        case 59: _t->slotRenameCollection(); break;
        case 60: _t->slotImageLocationMismatch(); break;
        case 61: _t->slotImageLocationChanged(); break;
        case 62: _t->slotToggleFullScreen(); break;
        case 63: _t->slotToggleMenuBarVisibility(); break;
        case 64: _t->slotToggleLayoutLock((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 65: _t->slotResetLayout(); break;
        case 66: _t->guiFactoryReset(); break;
        case 67: _t->showLog(); break;
        default: ;
        }
    }
}

const QMetaObject *Tellico::MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Tellico::MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int Tellico::MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 68)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 68;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 68)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 68;
    }
    return _id;
}
QT_WARNING_POP
