/****************************************************************************
** Meta object code from reading C++ file 'collectionfieldsdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../tellico-4.2.1/src/collectionfieldsdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'collectionfieldsdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Tellico22CollectionFieldsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto Tellico::CollectionFieldsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN7Tellico22CollectionFieldsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Tellico::CollectionFieldsDialog",
        "signalCollectionModified",
        "",
        "beginCommandGroup",
        "name",
        "endCommandGroup",
        "addField",
        "Tellico::Data::FieldPtr",
        "field",
        "modifyField",
        "removeField",
        "reorderFields",
        "Tellico::Data::FieldList",
        "fields",
        "slotOk",
        "slotApply",
        "slotDefault",
        "slotHelp",
        "slotNew",
        "slotDelete",
        "slotMoveUp",
        "slotMoveDown",
        "slotTypeChanged",
        "type",
        "slotHighlightedChanged",
        "index",
        "slotModified",
        "slotShowExtendedProperties",
        "slotSelectInitial",
        "slotDerivedChecked",
        "checked",
        "resetToCurrent"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalCollectionModified'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'beginCommandGroup'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'endCommandGroup'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addField'
        QtMocHelpers::SignalData<void(Tellico::Data::FieldPtr)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'modifyField'
        QtMocHelpers::SignalData<void(Tellico::Data::FieldPtr)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'removeField'
        QtMocHelpers::SignalData<void(Tellico::Data::FieldPtr)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'reorderFields'
        QtMocHelpers::SignalData<void(Tellico::Data::FieldList)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotOk'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDefault'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHelp'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNew'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDelete'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveUp'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveDown'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTypeChanged'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'slotHighlightedChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'slotModified'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowExtendedProperties'
        QtMocHelpers::SlotData<bool()>(27, 2, QMC::AccessPrivate, QMetaType::Bool),
        // Slot 'slotSelectInitial'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDerivedChecked'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Slot 'resetToCurrent'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CollectionFieldsDialog, qt_meta_tag_ZN7Tellico22CollectionFieldsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Tellico::CollectionFieldsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico22CollectionFieldsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico22CollectionFieldsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Tellico22CollectionFieldsDialogE_t>.metaTypes,
    nullptr
} };

void Tellico::CollectionFieldsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CollectionFieldsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalCollectionModified(); break;
        case 1: _t->beginCommandGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->endCommandGroup(); break;
        case 3: _t->addField((*reinterpret_cast<std::add_pointer_t<Tellico::Data::FieldPtr>>(_a[1]))); break;
        case 4: _t->modifyField((*reinterpret_cast<std::add_pointer_t<Tellico::Data::FieldPtr>>(_a[1]))); break;
        case 5: _t->removeField((*reinterpret_cast<std::add_pointer_t<Tellico::Data::FieldPtr>>(_a[1]))); break;
        case 6: _t->reorderFields((*reinterpret_cast<std::add_pointer_t<Tellico::Data::FieldList>>(_a[1]))); break;
        case 7: _t->slotOk(); break;
        case 8: _t->slotApply(); break;
        case 9: _t->slotDefault(); break;
        case 10: _t->slotHelp(); break;
        case 11: _t->slotNew(); break;
        case 12: _t->slotDelete(); break;
        case 13: _t->slotMoveUp(); break;
        case 14: _t->slotMoveDown(); break;
        case 15: _t->slotTypeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->slotHighlightedChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->slotModified(); break;
        case 18: { bool _r = _t->slotShowExtendedProperties();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->slotSelectInitial(); break;
        case 20: _t->slotDerivedChecked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->resetToCurrent(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Tellico::Data::FieldPtr >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Tellico::Data::FieldPtr >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Tellico::Data::FieldPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CollectionFieldsDialog::*)()>(_a, &CollectionFieldsDialog::signalCollectionModified, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFieldsDialog::*)(const QString & )>(_a, &CollectionFieldsDialog::beginCommandGroup, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFieldsDialog::*)()>(_a, &CollectionFieldsDialog::endCommandGroup, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFieldsDialog::*)(Tellico::Data::FieldPtr )>(_a, &CollectionFieldsDialog::addField, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFieldsDialog::*)(Tellico::Data::FieldPtr )>(_a, &CollectionFieldsDialog::modifyField, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFieldsDialog::*)(Tellico::Data::FieldPtr )>(_a, &CollectionFieldsDialog::removeField, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CollectionFieldsDialog::*)(Tellico::Data::FieldList )>(_a, &CollectionFieldsDialog::reorderFields, 6))
            return;
    }
}

const QMetaObject *Tellico::CollectionFieldsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Tellico::CollectionFieldsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico22CollectionFieldsDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Tellico::CollectionFieldsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void Tellico::CollectionFieldsDialog::signalCollectionModified()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Tellico::CollectionFieldsDialog::beginCommandGroup(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Tellico::CollectionFieldsDialog::endCommandGroup()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Tellico::CollectionFieldsDialog::addField(Tellico::Data::FieldPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Tellico::CollectionFieldsDialog::modifyField(Tellico::Data::FieldPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Tellico::CollectionFieldsDialog::removeField(Tellico::Data::FieldPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Tellico::CollectionFieldsDialog::reorderFields(Tellico::Data::FieldList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
