/***************************************************************************
    Copyright (C) 2021 Robby Stephenson <robby@periapsis.org>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License as        *
 *   published by the Free Software Foundation; either version 2 of        *
 *   the License or (at your option) version 3 or any later version        *
 *   accepted by the membership of KDE e.V. (or its successor approved     *
 *   by the membership of KDE e.V.), which shall act as a proxy            *
 *   defined in Section 14 of version 3 of the license.                    *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#include "fetchrequest.h"

using namespace Tellico;
using Tellico::Fetch::FetchRequest;

FetchRequest::FetchRequest()
    : m_collectionType(0)
    , m_key(Fetch::FetchFirst) {
}

FetchRequest::FetchRequest(Fetch::FetchKey key_, const QString& value_)
    : m_collectionType(0)
    , m_key(key_)
    , m_value(value_) {
}

FetchRequest::FetchRequest(int type_, Fetch::FetchKey key_, const QString& value_)
    : m_collectionType(type_)
    , m_key(key_)
    , m_value(value_) {
}

bool FetchRequest::isNull() const {
  return m_key == FetchFirst || m_value.isEmpty();
}
