/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.tellico.newstuff.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "newstuffadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class NewstuffAdaptor
 */

NewstuffAdaptor::NewstuffAdaptor(Tellico::NewStuff::Manager *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

NewstuffAdaptor::~NewstuffAdaptor()
{
    // destructor
}

bool NewstuffAdaptor::installScript(const QString &file)
{
    // handle method call org.kde.tellico.newstuff.installScript
    return parent()->installScript(file);
}

bool NewstuffAdaptor::installTemplate(const QString &file)
{
    // handle method call org.kde.tellico.newstuff.installTemplate
    return parent()->installTemplate(file);
}

bool NewstuffAdaptor::removeScript(const QString &file)
{
    // handle method call org.kde.tellico.newstuff.removeScript
    return parent()->removeScript(file);
}

bool NewstuffAdaptor::removeTemplate(const QString &file)
{
    // handle method call org.kde.tellico.newstuff.removeTemplate
    return parent()->removeTemplate(file);
}


#include "moc_newstuffadaptor.cpp"
