/****************************************************************************
** Meta object code from reading C++ file 'fetchdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../tellico-4.2.1/src/fetchdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fetchdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Tellico11FetchDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto Tellico::FetchDialog::qt_create_metaobjectdata<qt_meta_tag_ZN7Tellico11FetchDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Tellico::FetchDialog",
        "slotResetCollection",
        "",
        "slotSearchClicked",
        "slotClearClicked",
        "slotAddEntry",
        "slotMoreClicked",
        "slotShowEntry",
        "slotMoveProgress",
        "slotStatus",
        "status",
        "slotUpdateStatus",
        "slotFetchDone",
        "slotResultFound",
        "Tellico::Fetch::FetchResult*",
        "result",
        "slotKeyChanged",
        "slotSourceChanged",
        "source",
        "slotMultipleISBN",
        "toggle",
        "slotEditMultipleISBN",
        "slotInit",
        "slotLoadISBNList",
        "slotISBNTextChanged",
        "slotUPC2ISBN",
        "columnResized",
        "column",
        "slotBarcodeRecognized",
        "slotBarcodeGotImage",
        "QImage"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotResetCollection'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSearchClicked'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotClearClicked'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAddEntry'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoreClicked'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowEntry'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveProgress'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotStatus'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'slotUpdateStatus'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFetchDone'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotResultFound'
        QtMocHelpers::SlotData<void(Tellico::Fetch::FetchResult *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotKeyChanged'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotSourceChanged'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'slotMultipleISBN'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'slotEditMultipleISBN'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotInit'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLoadISBNList'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotISBNTextChanged'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUPC2ISBN'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'columnResized'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'slotBarcodeRecognized'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotBarcodeGotImage'
        QtMocHelpers::SlotData<void(const QImage &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FetchDialog, qt_meta_tag_ZN7Tellico11FetchDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Tellico::FetchDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico11FetchDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico11FetchDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Tellico11FetchDialogE_t>.metaTypes,
    nullptr
} };

void Tellico::FetchDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FetchDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotResetCollection(); break;
        case 1: _t->slotSearchClicked(); break;
        case 2: _t->slotClearClicked(); break;
        case 3: _t->slotAddEntry(); break;
        case 4: _t->slotMoreClicked(); break;
        case 5: _t->slotShowEntry(); break;
        case 6: _t->slotMoveProgress(); break;
        case 7: _t->slotStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->slotUpdateStatus(); break;
        case 9: _t->slotFetchDone(); break;
        case 10: _t->slotResultFound((*reinterpret_cast<std::add_pointer_t<Tellico::Fetch::FetchResult*>>(_a[1]))); break;
        case 11: _t->slotKeyChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotSourceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->slotMultipleISBN((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->slotEditMultipleISBN(); break;
        case 15: _t->slotInit(); break;
        case 16: _t->slotLoadISBNList(); break;
        case 17: _t->slotISBNTextChanged(); break;
        case 18: _t->slotUPC2ISBN(); break;
        case 19: _t->columnResized((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotBarcodeRecognized((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->slotBarcodeGotImage((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Tellico::FetchDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Tellico::FetchDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico11FetchDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Tellico::FetchDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}
QT_WARNING_POP
