/****************************************************************************
** Meta object code from reading C++ file 'document.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../tellico-4.2.1/src/document.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'document.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Tellico4Data8DocumentE_t {};
} // unnamed namespace

template <> constexpr inline auto Tellico::Data::Document::qt_create_metaobjectdata<qt_meta_tag_ZN7Tellico4Data8DocumentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Tellico::Data::Document",
        "signalModified",
        "",
        "modified",
        "signalStatusMsg",
        "str",
        "signalCollectionImagesLoaded",
        "Tellico::Data::CollPtr",
        "coll",
        "signalCollectionAdded",
        "signalCollectionDeleted",
        "signalCollectionModified",
        "structuralChange",
        "slotSetModified",
        "slotSetClean",
        "clean",
        "slotLoadAllImages"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalModified'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'signalStatusMsg'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'signalCollectionImagesLoaded'
        QtMocHelpers::SignalData<void(Tellico::Data::CollPtr)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'signalCollectionAdded'
        QtMocHelpers::SignalData<void(Tellico::Data::CollPtr)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'signalCollectionDeleted'
        QtMocHelpers::SignalData<void(Tellico::Data::CollPtr)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'signalCollectionModified'
        QtMocHelpers::SignalData<void(Tellico::Data::CollPtr, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::Bool, 12 },
        }}),
        // Slot 'slotSetModified'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetClean'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotLoadAllImages'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Document, qt_meta_tag_ZN7Tellico4Data8DocumentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Tellico::Data::Document::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico4Data8DocumentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico4Data8DocumentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Tellico4Data8DocumentE_t>.metaTypes,
    nullptr
} };

void Tellico::Data::Document::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Document *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->signalStatusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->signalCollectionImagesLoaded((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 3: _t->signalCollectionAdded((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 4: _t->signalCollectionDeleted((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 5: _t->signalCollectionModified((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->slotSetModified(); break;
        case 7: _t->slotSetClean((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->slotLoadAllImages(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Document::*)(bool )>(_a, &Document::signalModified, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const QString & )>(_a, &Document::signalStatusMsg, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(Tellico::Data::CollPtr )>(_a, &Document::signalCollectionImagesLoaded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(Tellico::Data::CollPtr )>(_a, &Document::signalCollectionAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(Tellico::Data::CollPtr )>(_a, &Document::signalCollectionDeleted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(Tellico::Data::CollPtr , bool )>(_a, &Document::signalCollectionModified, 5))
            return;
    }
}

const QMetaObject *Tellico::Data::Document::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Tellico::Data::Document::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico4Data8DocumentE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Tellico::Data::Document::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Tellico::Data::Document::signalModified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Tellico::Data::Document::signalStatusMsg(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Tellico::Data::Document::signalCollectionImagesLoaded(Tellico::Data::CollPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Tellico::Data::Document::signalCollectionAdded(Tellico::Data::CollPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Tellico::Data::Document::signalCollectionDeleted(Tellico::Data::CollPtr _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Tellico::Data::Document::signalCollectionModified(Tellico::Data::CollPtr _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
QT_WARNING_POP
