/********************************************************************************
** Form generated from reading UI file 'collectiontemplatedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLLECTIONTEMPLATEDIALOG_H
#define UI_COLLECTIONTEMPLATEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "kiconbutton.h"

QT_BEGIN_NAMESPACE

class Ui_CollectionTemplateDialog
{
public:
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *labelName;
    QLineEdit *editName;
    QLabel *labelComment;
    QLineEdit *editComment;
    KIconButton *editIcon;
    QLabel *labelIcon;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *CollectionTemplateDialog)
    {
        if (CollectionTemplateDialog->objectName().isEmpty())
            CollectionTemplateDialog->setObjectName("CollectionTemplateDialog");
        CollectionTemplateDialog->resize(250, 158);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CollectionTemplateDialog->sizePolicy().hasHeightForWidth());
        CollectionTemplateDialog->setSizePolicy(sizePolicy);
        CollectionTemplateDialog->setModal(true);
        layoutWidget = new QWidget(CollectionTemplateDialog);
        layoutWidget->setObjectName("layoutWidget");
        layoutWidget->setGeometry(QRect(0, 0, 242, 152));
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setContentsMargins(6, -1, 6, -1);
        labelName = new QLabel(layoutWidget);
        labelName->setObjectName("labelName");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, labelName);

        editName = new QLineEdit(layoutWidget);
        editName->setObjectName("editName");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, editName);

        labelComment = new QLabel(layoutWidget);
        labelComment->setObjectName("labelComment");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, labelComment);

        editComment = new QLineEdit(layoutWidget);
        editComment->setObjectName("editComment");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, editComment);

        editIcon = new KIconButton(layoutWidget);
        editIcon->setObjectName("editIcon");
        editIcon->setIconSize(32);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, editIcon);

        labelIcon = new QLabel(layoutWidget);
        labelIcon->setObjectName("labelIcon");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, labelIcon);


        verticalLayout->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(10, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(layoutWidget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(CollectionTemplateDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, CollectionTemplateDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, CollectionTemplateDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(CollectionTemplateDialog);
    } // setupUi

    void retranslateUi(QDialog *CollectionTemplateDialog)
    {
        labelName->setText(QCoreApplication::translate("CollectionTemplateDialog", "Template Name:", nullptr));
        labelComment->setText(QCoreApplication::translate("CollectionTemplateDialog", "Comment:", nullptr));
        labelIcon->setText(QCoreApplication::translate("CollectionTemplateDialog", "Icon:", nullptr));
        (void)CollectionTemplateDialog;
    } // retranslateUi

};

namespace Ui {
    class CollectionTemplateDialog: public Ui_CollectionTemplateDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COLLECTIONTEMPLATEDIALOG_H
