/***************************************************************************
    Copyright (C) 2005-2009 Robby Stephenson <robby@periapsis.org>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License as        *
 *   published by the Free Software Foundation; either version 2 of        *
 *   the License or (at your option) version 3 or any later version        *
 *   accepted by the membership of KDE e.V. (or its successor approved     *
 *   by the membership of KDE e.V.), which shall act as a proxy            *
 *   defined in Section 14 of version 3 of the license.                    *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#ifndef TELLICO_OBSERVER_H
#define TELLICO_OBSERVER_H

#include "datavectors.h"

namespace Tellico {
  class Filter;

/**
 * @author Robby Stephenson
 */
class Observer {

public:
  Observer() {}
  virtual ~Observer() {}

  virtual void    addBorrower(Data::BorrowerPtr) {}
  virtual void modifyBorrower(Data::BorrowerPtr) {}
  virtual void removeBorrower(Data::BorrowerPtr) {}

  virtual void    addEntries(Data::EntryList) {}
  virtual void modifyEntries(Data::EntryList) {}
  virtual void removeEntries(Data::EntryList) {}

  virtual void    addField(Data::CollPtr, Data::FieldPtr) {}
  // coll, oldfield, newfield
  virtual void modifyField(Data::CollPtr, Data::FieldPtr, Data::FieldPtr) {}
  virtual void removeField(Data::CollPtr, Data::FieldPtr) {}

  virtual void    addFilter(FilterPtr) {}
  virtual void modifyFilter(FilterPtr) {}
  virtual void removeFilter(FilterPtr) {}

private:
  Q_DISABLE_COPY(Observer)
};

}

#endif
