/****************************************************************************
** Meta object code from reading C++ file 'controller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../tellico-4.2.1/src/controller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'controller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Tellico10ControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto Tellico::Controller::qt_create_metaobjectdata<qt_meta_tag_ZN7Tellico10ControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Tellico::Controller",
        "collectionAdded",
        "",
        "collType",
        "slotCollectionAdded",
        "Tellico::Data::CollPtr",
        "coll",
        "slotCollectionModified",
        "structuralChange",
        "slotCollectionDeleted",
        "slotFieldAdded",
        "Tellico::Data::FieldPtr",
        "field",
        "slotRefreshField",
        "slotClearSelection",
        "slotUpdateSelection",
        "Tellico::Data::EntryList",
        "entries",
        "slotCopySelectedEntries",
        "slotUpdateSelectedEntries",
        "source",
        "slotDeleteSelectedEntries",
        "slotMergeSelectedEntries",
        "slotUpdateFilter",
        "Tellico::FilterPtr",
        "filter",
        "slotCheckOut",
        "slotCheckIn"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'collectionAdded'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'slotCollectionAdded'
        QtMocHelpers::SlotData<void(Tellico::Data::CollPtr)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotCollectionModified'
        QtMocHelpers::SlotData<void(Tellico::Data::CollPtr, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotCollectionDeleted'
        QtMocHelpers::SlotData<void(Tellico::Data::CollPtr)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotFieldAdded'
        QtMocHelpers::SlotData<void(Tellico::Data::CollPtr, Tellico::Data::FieldPtr)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'slotRefreshField'
        QtMocHelpers::SlotData<void(Tellico::Data::FieldPtr)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'slotClearSelection'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdateSelection'
        QtMocHelpers::SlotData<void(const Tellico::Data::EntryList &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotCopySelectedEntries'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdateSelectedEntries'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotDeleteSelectedEntries'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMergeSelectedEntries'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdateFilter'
        QtMocHelpers::SlotData<void(Tellico::FilterPtr)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'slotCheckOut'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCheckIn'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCheckIn'
        QtMocHelpers::SlotData<void(const Tellico::Data::EntryList &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Controller, qt_meta_tag_ZN7Tellico10ControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Tellico::Controller::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico10ControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico10ControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Tellico10ControllerE_t>.metaTypes,
    nullptr
} };

void Tellico::Controller::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Controller *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->collectionAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->slotCollectionAdded((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 2: _t->slotCollectionModified((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->slotCollectionDeleted((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1]))); break;
        case 4: _t->slotFieldAdded((*reinterpret_cast<std::add_pointer_t<Tellico::Data::CollPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Tellico::Data::FieldPtr>>(_a[2]))); break;
        case 5: _t->slotRefreshField((*reinterpret_cast<std::add_pointer_t<Tellico::Data::FieldPtr>>(_a[1]))); break;
        case 6: _t->slotClearSelection(); break;
        case 7: _t->slotUpdateSelection((*reinterpret_cast<std::add_pointer_t<Tellico::Data::EntryList>>(_a[1]))); break;
        case 8: _t->slotCopySelectedEntries(); break;
        case 9: _t->slotUpdateSelectedEntries((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->slotDeleteSelectedEntries(); break;
        case 11: _t->slotMergeSelectedEntries(); break;
        case 12: _t->slotUpdateFilter((*reinterpret_cast<std::add_pointer_t<Tellico::FilterPtr>>(_a[1]))); break;
        case 13: _t->slotCheckOut(); break;
        case 14: _t->slotCheckIn(); break;
        case 15: _t->slotCheckIn((*reinterpret_cast<std::add_pointer_t<Tellico::Data::EntryList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Tellico::Data::FieldPtr >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Tellico::Data::FieldPtr >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Controller::*)(int )>(_a, &Controller::collectionAdded, 0))
            return;
    }
}

const QMetaObject *Tellico::Controller::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Tellico::Controller::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico10ControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Tellico::Controller::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void Tellico::Controller::collectionAdded(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
