// This file is generated by kconfig_compiler_kf6 from tellico_config.kcfg.
// All changes you do to this file will be lost.

#include "tellico_config.h"

#include <KColorScheme>
#include <KLocalizedString>
#include <QFontDatabase>
#include <qglobal.h>
#include <QFile>

using namespace Tellico;

namespace Tellico {

class ConfigHelper
{
  public:
    ConfigHelper() noexcept : q(nullptr) {}
    ~ConfigHelper() noexcept { delete q; q = nullptr; }
    ConfigHelper(const ConfigHelper&) = delete;
    ConfigHelper& operator=(const ConfigHelper&) = delete;
    Config *q;
};

}
Q_GLOBAL_STATIC(ConfigHelper, s_globalConfig)
Config *Config::self()
{
  if (!s_globalConfig()->q) {
    new Config;
    s_globalConfig()->q->read();
  }

  return s_globalConfig()->q;
}

Config::Config( )
  : KConfigSkeleton( QStringLiteral( "tellicorc" ) )
{
  Q_ASSERT(!s_globalConfig()->q);
  s_globalConfig()->q = this;
  setCurrentGroup( QStringLiteral( "Group View Options" ) );

  KConfigSkeleton::ItemInt  *itemGroupViewSortColumn;
  itemGroupViewSortColumn = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "SortColumn" ), mGroupViewSortColumn, 0 );
  addItem( itemGroupViewSortColumn, QStringLiteral( "groupViewSortColumn" ) );
  KConfigSkeleton::ItemBool  *itemGroupViewSortAscending;
  itemGroupViewSortAscending = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SortAscending" ), mGroupViewSortAscending, true );
  addItem( itemGroupViewSortAscending, QStringLiteral( "groupViewSortAscending" ) );

  setCurrentGroup( QStringLiteral( "Filter View Options" ) );

  KConfigSkeleton::ItemInt  *itemFilterViewSortColumn;
  itemFilterViewSortColumn = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "SortColumn" ), mFilterViewSortColumn, 0 );
  addItem( itemFilterViewSortColumn, QStringLiteral( "filterViewSortColumn" ) );
  KConfigSkeleton::ItemBool  *itemFilterViewSortAscending;
  itemFilterViewSortAscending = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SortAscending" ), mFilterViewSortAscending, true );
  addItem( itemFilterViewSortAscending, QStringLiteral( "filterViewSortAscending" ) );

  setCurrentGroup( QStringLiteral( "Loan View Options" ) );

  KConfigSkeleton::ItemInt  *itemLoanViewSortColumn;
  itemLoanViewSortColumn = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "SortColumn" ), mLoanViewSortColumn, 0 );
  addItem( itemLoanViewSortColumn, QStringLiteral( "loanViewSortColumn" ) );
  KConfigSkeleton::ItemBool  *itemLoanViewSortAscending;
  itemLoanViewSortAscending = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SortAscending" ), mLoanViewSortAscending, true );
  addItem( itemLoanViewSortAscending, QStringLiteral( "loanViewSortAscending" ) );

  setCurrentGroup( QStringLiteral( "Export Options - Bibtex" ) );

  KConfigSkeleton::ItemBool  *itemUseBraces;
  itemUseBraces = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Use Braces" ), mUseBraces, true );
  addItem( itemUseBraces, QStringLiteral( "UseBraces" ) );

  setCurrentGroup( QStringLiteral( "General Options" ) );

  KConfigSkeleton::ItemBool  *itemShowEditWidget;
  itemShowEditWidget = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Show Edit Widget" ), mShowEditWidget, false );
  addItem( itemShowEditWidget, QStringLiteral( "ShowEditWidget" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesViewWidget;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ListView");
    valuesViewWidget.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("IconView");
    valuesViewWidget.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemViewWidget;
  itemViewWidget = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "View Widget" ), mViewWidget, valuesViewWidget, ListView );
  addItem( itemViewWidget, QStringLiteral( "ViewWidget" ) );
  KConfigSkeleton::ItemBool  *itemLockLayout;
  itemLockLayout = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Lock Layout" ), mLockLayout, true );
  addItem( itemLockLayout, QStringLiteral( "LockLayout" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesImageLocation;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ImagesInFile");
    valuesImageLocation.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ImagesInAppDir");
    valuesImageLocation.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ImagesInLocalDir");
    valuesImageLocation.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemImageLocation;
  itemImageLocation = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Image Location" ), mImageLocation, valuesImageLocation, ImagesInFile );
  addItem( itemImageLocation, QStringLiteral( "ImageLocation" ) );
  KConfigSkeleton::ItemBool  *itemAskWriteImagesInFile;
  itemAskWriteImagesInFile = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Ask Write Images In File" ), mAskWriteImagesInFile, true );
  addItem( itemAskWriteImagesInFile, QStringLiteral( "AskWriteImagesInFile" ) );
  KConfigSkeleton::ItemBool  *itemReopenLastFile;
  itemReopenLastFile = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Reopen Last File" ), mReopenLastFile, true );
  addItem( itemReopenLastFile, QStringLiteral( "ReopenLastFile" ) );
  KConfigSkeleton::ItemBool  *itemAutoCapitalization;
  itemAutoCapitalization = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Auto Capitalization" ), mAutoCapitalization, true );
  addItem( itemAutoCapitalization, QStringLiteral( "AutoCapitalization" ) );
  KConfigSkeleton::ItemBool  *itemAutoFormat;
  itemAutoFormat = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Auto Format" ), mAutoFormat, true );
  addItem( itemAutoFormat, QStringLiteral( "AutoFormat" ) );
  KConfigSkeleton::ItemString  *itemLastOpenFile;
  itemLastOpenFile = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Last Open File" ), mLastOpenFile );
  addItem( itemLastOpenFile, QStringLiteral( "LastOpenFile" ) );
  KConfigSkeleton::ItemString  *itemNoCapitalizationString;
  itemNoCapitalizationString = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "No Capitalization" ), mNoCapitalizationString, i18nc("comma-separated list of words ignored by auto-capitalize", "a,an,and,as,at,but,by,for,from,in,into,nor,of,off,on,onto,or,out,over,the,to,up,with") );
  addItem( itemNoCapitalizationString, QStringLiteral( "noCapitalizationString" ) );
  KConfigSkeleton::ItemString  *itemArticlesString;
  itemArticlesString = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Articles" ), mArticlesString, i18nc("comma-separated list of articles ignored for title sorting", "the") );
  addItem( itemArticlesString, QStringLiteral( "articlesString" ) );
  KConfigSkeleton::ItemString  *itemNameSuffixesString;
  itemNameSuffixesString = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Name Suffixes" ), mNameSuffixesString, i18nc("comma-separated list of common surname suffixes, used for sorting", "jr.,jr,iii,iv") );
  addItem( itemNameSuffixesString, QStringLiteral( "nameSuffixesString" ) );
  KConfigSkeleton::ItemString  *itemSurnamePrefixesString;
  itemSurnamePrefixesString = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Surname Prefixes" ), mSurnamePrefixesString, i18nc("comma-separated list of common surname prefixes, used for sorting", "de,van,der,van der,von") );
  addItem( itemSurnamePrefixesString, QStringLiteral( "surnamePrefixesString" ) );
  KConfigSkeleton::ItemInt  *itemMaxIconSize;
  itemMaxIconSize = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Max Icon Size" ), mMaxIconSize, 96 );
  addItem( itemMaxIconSize, QStringLiteral( "MaxIconSize" ) );
  KConfigSkeleton::ItemInt  *itemIconCacheSize;
  itemIconCacheSize = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Icon Cache Size" ), mIconCacheSize, 1000 );
  addItem( itemIconCacheSize, QStringLiteral( "IconCacheSize" ) );
  KConfigSkeleton::ItemULongLong  *itemImageCacheSize;
  itemImageCacheSize = new KConfigSkeleton::ItemULongLong( currentGroup(), QStringLiteral( "Image Cache Size" ), mImageCacheSize, (512 * 1024 * 1024) );
  addItem( itemImageCacheSize, QStringLiteral( "ImageCacheSize" ) );
  KConfigSkeleton::ItemInt  *itemMaxCustomURLSettings;
  itemMaxCustomURLSettings = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Max Custom URL Settings" ), mMaxCustomURLSettings, 9 );
  addItem( itemMaxCustomURLSettings, QStringLiteral( "MaxCustomURLSettings" ) );
  KConfigSkeleton::ItemBool  *itemEnableWebcam;
  itemEnableWebcam = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Enable Webcam" ), mEnableWebcam, true );
  addItem( itemEnableWebcam, QStringLiteral( "EnableWebcam" ) );
  KConfigSkeleton::ItemBool  *itemQuickFilterRegExp;
  itemQuickFilterRegExp = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Quick Filter RegExp" ), mQuickFilterRegExp, false );
  addItem( itemQuickFilterRegExp, QStringLiteral( "QuickFilterRegExp" ) );
  KConfigSkeleton::ItemBool  *itemShowWelcome;
  itemShowWelcome = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Show Welcome" ), mShowWelcome, true );
  addItem( itemShowWelcome, QStringLiteral( "ShowWelcome" ) );

  setCurrentGroup( QStringLiteral( "Printing" ) );

  KConfigSkeleton::ItemBool  *itemPrintFieldHeaders;
  itemPrintFieldHeaders = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Print Field Headers" ), mPrintFieldHeaders, true );
  addItem( itemPrintFieldHeaders, QStringLiteral( "PrintFieldHeaders" ) );
  KConfigSkeleton::ItemBool  *itemPrintFormatted;
  itemPrintFormatted = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Print Formatted" ), mPrintFormatted, true );
  addItem( itemPrintFormatted, QStringLiteral( "PrintFormatted" ) );
  KConfigSkeleton::ItemBool  *itemPrintGrouped;
  itemPrintGrouped = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Print Grouped" ), mPrintGrouped, true );
  addItem( itemPrintGrouped, QStringLiteral( "PrintGrouped" ) );
  KConfigSkeleton::ItemInt  *itemMaxImageWidth;
  itemMaxImageWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Max Image Width" ), mMaxImageWidth, 50 );
  addItem( itemMaxImageWidth, QStringLiteral( "MaxImageWidth" ) );
  KConfigSkeleton::ItemInt  *itemMaxImageHeight;
  itemMaxImageHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "Max Image Height" ), mMaxImageHeight, 50 );
  addItem( itemMaxImageHeight, QStringLiteral( "MaxImageHeight" ) );

  setCurrentGroup( QStringLiteral( "Options - book" ) );

  KConfigSkeleton::ItemString  *itemTemplateBook;
  itemTemplateBook = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateBook, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateBook, QStringLiteral( "templateBook" ) );
  KConfigSkeleton::ItemFont  *itemFontBook;
  itemFontBook = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontBook, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontBook, QStringLiteral( "fontBook" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorBook;
  itemBaseColorBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorBook, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorBook, QStringLiteral( "baseColorBook" ) );
  KConfigSkeleton::ItemColor  *itemTextColorBook;
  itemTextColorBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorBook, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorBook, QStringLiteral( "textColorBook" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorBook;
  itemHighlightedBaseColorBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorBook, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorBook, QStringLiteral( "highlightedBaseColorBook" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorBook;
  itemHighlightedTextColorBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorBook, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorBook, QStringLiteral( "highlightedTextColorBook" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorBook;
  itemLinkColorBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorBook, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorBook, QStringLiteral( "linkColorBook" ) );

  setCurrentGroup( QStringLiteral( "Options - video" ) );

  KConfigSkeleton::ItemString  *itemTemplateVideo;
  itemTemplateVideo = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateVideo, QStringLiteral( "Video" ) );
  addItem( itemTemplateVideo, QStringLiteral( "templateVideo" ) );
  KConfigSkeleton::ItemFont  *itemFontVideo;
  itemFontVideo = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontVideo, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontVideo, QStringLiteral( "fontVideo" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorVideo;
  itemBaseColorVideo = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorVideo, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorVideo, QStringLiteral( "baseColorVideo" ) );
  KConfigSkeleton::ItemColor  *itemTextColorVideo;
  itemTextColorVideo = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorVideo, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorVideo, QStringLiteral( "textColorVideo" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorVideo;
  itemHighlightedBaseColorVideo = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorVideo, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorVideo, QStringLiteral( "highlightedBaseColorVideo" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorVideo;
  itemHighlightedTextColorVideo = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorVideo, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorVideo, QStringLiteral( "highlightedTextColorVideo" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorVideo;
  itemLinkColorVideo = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorVideo, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorVideo, QStringLiteral( "linkColorVideo" ) );

  setCurrentGroup( QStringLiteral( "Options - album" ) );

  KConfigSkeleton::ItemString  *itemTemplateAlbum;
  itemTemplateAlbum = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateAlbum, QStringLiteral( "Album" ) );
  addItem( itemTemplateAlbum, QStringLiteral( "templateAlbum" ) );
  KConfigSkeleton::ItemFont  *itemFontAlbum;
  itemFontAlbum = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontAlbum, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontAlbum, QStringLiteral( "fontAlbum" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorAlbum;
  itemBaseColorAlbum = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorAlbum, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorAlbum, QStringLiteral( "baseColorAlbum" ) );
  KConfigSkeleton::ItemColor  *itemTextColorAlbum;
  itemTextColorAlbum = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorAlbum, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorAlbum, QStringLiteral( "textColorAlbum" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorAlbum;
  itemHighlightedBaseColorAlbum = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorAlbum, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorAlbum, QStringLiteral( "highlightedBaseColorAlbum" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorAlbum;
  itemHighlightedTextColorAlbum = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorAlbum, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorAlbum, QStringLiteral( "highlightedTextColorAlbum" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorAlbum;
  itemLinkColorAlbum = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorAlbum, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorAlbum, QStringLiteral( "linkColorAlbum" ) );

  setCurrentGroup( QStringLiteral( "Options - bibtex" ) );

  KConfigSkeleton::ItemPath  *itemLyxpipe;
  itemLyxpipe = new KConfigSkeleton::ItemPath( currentGroup(), QStringLiteral( "lyxpipe" ), mLyxpipe, QStringLiteral( "$HOME/.lyx/lyxpipe" ) );
  addItem( itemLyxpipe, QStringLiteral( "lyxpipe" ) );
  KConfigSkeleton::ItemString  *itemTemplateBibtex;
  itemTemplateBibtex = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateBibtex, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateBibtex, QStringLiteral( "templateBibtex" ) );
  KConfigSkeleton::ItemFont  *itemFontBibtex;
  itemFontBibtex = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontBibtex, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontBibtex, QStringLiteral( "fontBibtex" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorBibtex;
  itemBaseColorBibtex = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorBibtex, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorBibtex, QStringLiteral( "baseColorBibtex" ) );
  KConfigSkeleton::ItemColor  *itemTextColorBibtex;
  itemTextColorBibtex = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorBibtex, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorBibtex, QStringLiteral( "textColorBibtex" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorBibtex;
  itemHighlightedBaseColorBibtex = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorBibtex, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorBibtex, QStringLiteral( "highlightedBaseColorBibtex" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorBibtex;
  itemHighlightedTextColorBibtex = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorBibtex, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorBibtex, QStringLiteral( "highlightedTextColorBibtex" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorBibtex;
  itemLinkColorBibtex = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorBibtex, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorBibtex, QStringLiteral( "linkColorBibtex" ) );

  setCurrentGroup( QStringLiteral( "Options - comic" ) );

  KConfigSkeleton::ItemString  *itemTemplateComicBook;
  itemTemplateComicBook = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateComicBook, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateComicBook, QStringLiteral( "templateComicBook" ) );
  KConfigSkeleton::ItemFont  *itemFontComicBook;
  itemFontComicBook = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontComicBook, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontComicBook, QStringLiteral( "fontComicBook" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorComicBook;
  itemBaseColorComicBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorComicBook, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorComicBook, QStringLiteral( "baseColorComicBook" ) );
  KConfigSkeleton::ItemColor  *itemTextColorComicBook;
  itemTextColorComicBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorComicBook, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorComicBook, QStringLiteral( "textColorComicBook" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorComicBook;
  itemHighlightedBaseColorComicBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorComicBook, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorComicBook, QStringLiteral( "highlightedBaseColorComicBook" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorComicBook;
  itemHighlightedTextColorComicBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorComicBook, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorComicBook, QStringLiteral( "highlightedTextColorComicBook" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorComicBook;
  itemLinkColorComicBook = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorComicBook, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorComicBook, QStringLiteral( "linkColorComicBook" ) );

  setCurrentGroup( QStringLiteral( "Options - wine" ) );

  KConfigSkeleton::ItemString  *itemTemplateWine;
  itemTemplateWine = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateWine, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateWine, QStringLiteral( "templateWine" ) );
  KConfigSkeleton::ItemFont  *itemFontWine;
  itemFontWine = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontWine, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontWine, QStringLiteral( "fontWine" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorWine;
  itemBaseColorWine = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorWine, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorWine, QStringLiteral( "baseColorWine" ) );
  KConfigSkeleton::ItemColor  *itemTextColorWine;
  itemTextColorWine = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorWine, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorWine, QStringLiteral( "textColorWine" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorWine;
  itemHighlightedBaseColorWine = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorWine, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorWine, QStringLiteral( "highlightedBaseColorWine" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorWine;
  itemHighlightedTextColorWine = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorWine, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorWine, QStringLiteral( "highlightedTextColorWine" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorWine;
  itemLinkColorWine = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorWine, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorWine, QStringLiteral( "linkColorWine" ) );

  setCurrentGroup( QStringLiteral( "Options - coin" ) );

  KConfigSkeleton::ItemString  *itemTemplateCoin;
  itemTemplateCoin = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateCoin, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateCoin, QStringLiteral( "templateCoin" ) );
  KConfigSkeleton::ItemFont  *itemFontCoin;
  itemFontCoin = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontCoin, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontCoin, QStringLiteral( "fontCoin" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorCoin;
  itemBaseColorCoin = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorCoin, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorCoin, QStringLiteral( "baseColorCoin" ) );
  KConfigSkeleton::ItemColor  *itemTextColorCoin;
  itemTextColorCoin = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorCoin, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorCoin, QStringLiteral( "textColorCoin" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorCoin;
  itemHighlightedBaseColorCoin = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorCoin, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorCoin, QStringLiteral( "highlightedBaseColorCoin" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorCoin;
  itemHighlightedTextColorCoin = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorCoin, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorCoin, QStringLiteral( "highlightedTextColorCoin" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorCoin;
  itemLinkColorCoin = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorCoin, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorCoin, QStringLiteral( "linkColorCoin" ) );

  setCurrentGroup( QStringLiteral( "Options - stamp" ) );

  KConfigSkeleton::ItemString  *itemTemplateStamp;
  itemTemplateStamp = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateStamp, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateStamp, QStringLiteral( "templateStamp" ) );
  KConfigSkeleton::ItemFont  *itemFontStamp;
  itemFontStamp = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontStamp, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontStamp, QStringLiteral( "fontStamp" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorStamp;
  itemBaseColorStamp = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorStamp, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorStamp, QStringLiteral( "baseColorStamp" ) );
  KConfigSkeleton::ItemColor  *itemTextColorStamp;
  itemTextColorStamp = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorStamp, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorStamp, QStringLiteral( "textColorStamp" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorStamp;
  itemHighlightedBaseColorStamp = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorStamp, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorStamp, QStringLiteral( "highlightedBaseColorStamp" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorStamp;
  itemHighlightedTextColorStamp = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorStamp, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorStamp, QStringLiteral( "highlightedTextColorStamp" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorStamp;
  itemLinkColorStamp = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorStamp, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorStamp, QStringLiteral( "linkColorStamp" ) );

  setCurrentGroup( QStringLiteral( "Options - card" ) );

  KConfigSkeleton::ItemString  *itemTemplateCard;
  itemTemplateCard = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateCard, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateCard, QStringLiteral( "templateCard" ) );
  KConfigSkeleton::ItemFont  *itemFontCard;
  itemFontCard = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontCard, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontCard, QStringLiteral( "fontCard" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorCard;
  itemBaseColorCard = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorCard, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorCard, QStringLiteral( "baseColorCard" ) );
  KConfigSkeleton::ItemColor  *itemTextColorCard;
  itemTextColorCard = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorCard, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorCard, QStringLiteral( "textColorCard" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorCard;
  itemHighlightedBaseColorCard = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorCard, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorCard, QStringLiteral( "highlightedBaseColorCard" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorCard;
  itemHighlightedTextColorCard = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorCard, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorCard, QStringLiteral( "highlightedTextColorCard" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorCard;
  itemLinkColorCard = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorCard, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorCard, QStringLiteral( "linkColorCard" ) );

  setCurrentGroup( QStringLiteral( "Options - game" ) );

  KConfigSkeleton::ItemString  *itemTemplateGame;
  itemTemplateGame = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateGame, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateGame, QStringLiteral( "templateGame" ) );
  KConfigSkeleton::ItemFont  *itemFontGame;
  itemFontGame = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontGame, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontGame, QStringLiteral( "fontGame" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorGame;
  itemBaseColorGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorGame, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorGame, QStringLiteral( "baseColorGame" ) );
  KConfigSkeleton::ItemColor  *itemTextColorGame;
  itemTextColorGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorGame, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorGame, QStringLiteral( "textColorGame" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorGame;
  itemHighlightedBaseColorGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorGame, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorGame, QStringLiteral( "highlightedBaseColorGame" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorGame;
  itemHighlightedTextColorGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorGame, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorGame, QStringLiteral( "highlightedTextColorGame" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorGame;
  itemLinkColorGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorGame, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorGame, QStringLiteral( "linkColorGame" ) );

  setCurrentGroup( QStringLiteral( "Options - file" ) );

  KConfigSkeleton::ItemString  *itemTemplateFile;
  itemTemplateFile = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateFile, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateFile, QStringLiteral( "templateFile" ) );
  KConfigSkeleton::ItemFont  *itemFontFile;
  itemFontFile = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontFile, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontFile, QStringLiteral( "fontFile" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorFile;
  itemBaseColorFile = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorFile, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorFile, QStringLiteral( "baseColorFile" ) );
  KConfigSkeleton::ItemColor  *itemTextColorFile;
  itemTextColorFile = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorFile, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorFile, QStringLiteral( "textColorFile" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorFile;
  itemHighlightedBaseColorFile = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorFile, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorFile, QStringLiteral( "highlightedBaseColorFile" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorFile;
  itemHighlightedTextColorFile = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorFile, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorFile, QStringLiteral( "highlightedTextColorFile" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorFile;
  itemLinkColorFile = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorFile, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorFile, QStringLiteral( "linkColorFile" ) );

  setCurrentGroup( QStringLiteral( "Options - boardgame" ) );

  KConfigSkeleton::ItemString  *itemTemplateBoardGame;
  itemTemplateBoardGame = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateBoardGame, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateBoardGame, QStringLiteral( "templateBoardGame" ) );
  KConfigSkeleton::ItemFont  *itemFontBoardGame;
  itemFontBoardGame = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontBoardGame, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontBoardGame, QStringLiteral( "fontBoardGame" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorBoardGame;
  itemBaseColorBoardGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorBoardGame, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorBoardGame, QStringLiteral( "baseColorBoardGame" ) );
  KConfigSkeleton::ItemColor  *itemTextColorBoardGame;
  itemTextColorBoardGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorBoardGame, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorBoardGame, QStringLiteral( "textColorBoardGame" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorBoardGame;
  itemHighlightedBaseColorBoardGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorBoardGame, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorBoardGame, QStringLiteral( "highlightedBaseColorBoardGame" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorBoardGame;
  itemHighlightedTextColorBoardGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorBoardGame, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorBoardGame, QStringLiteral( "highlightedTextColorBoardGame" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorBoardGame;
  itemLinkColorBoardGame = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorBoardGame, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorBoardGame, QStringLiteral( "linkColorBoardGame" ) );

  setCurrentGroup( QStringLiteral( "Options - entry" ) );

  KConfigSkeleton::ItemString  *itemTemplateBase;
  itemTemplateBase = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Entry Template" ), mTemplateBase, QStringLiteral( "Fancy" ) );
  addItem( itemTemplateBase, QStringLiteral( "templateBase" ) );
  KConfigSkeleton::ItemFont  *itemFontBase;
  itemFontBase = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "Template Font" ), mFontBase, QFontDatabase::systemFont(QFontDatabase::GeneralFont) );
  addItem( itemFontBase, QStringLiteral( "fontBase" ) );
  KConfigSkeleton::ItemColor  *itemBaseColorBase;
  itemBaseColorBase = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Base Color" ), mBaseColorBase, KColorScheme(QPalette::Active).background().color() );
  addItem( itemBaseColorBase, QStringLiteral( "baseColorBase" ) );
  KConfigSkeleton::ItemColor  *itemTextColorBase;
  itemTextColorBase = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Text Color" ), mTextColorBase, KColorScheme(QPalette::Active).foreground().color() );
  addItem( itemTextColorBase, QStringLiteral( "textColorBase" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedBaseColorBase;
  itemHighlightedBaseColorBase = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlight Color" ), mHighlightedBaseColorBase, KColorScheme(QPalette::Active, KColorScheme::Selection).background().color() );
  addItem( itemHighlightedBaseColorBase, QStringLiteral( "highlightedBaseColorBase" ) );
  KConfigSkeleton::ItemColor  *itemHighlightedTextColorBase;
  itemHighlightedTextColorBase = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Highlighted Text Color" ), mHighlightedTextColorBase, KColorScheme(QPalette::Active, KColorScheme::Selection).foreground().color() );
  addItem( itemHighlightedTextColorBase, QStringLiteral( "highlightedTextColorBase" ) );
  KConfigSkeleton::ItemColor  *itemLinkColorBase;
  itemLinkColorBase = new KConfigSkeleton::ItemColor( currentGroup(), QStringLiteral( "Template Link Color" ), mLinkColorBase, KColorScheme(QPalette::Active).foreground(KColorScheme::LinkText).color() );
  addItem( itemLinkColorBase, QStringLiteral( "linkColorBase" ) );
}

Config::~Config()
{
  if (s_globalConfig.exists() && !s_globalConfig.isDestroyed()) {
    s_globalConfig()->q = nullptr;
  }
}

