/****************************************************************************
** Meta object code from reading C++ file 'tablefieldwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../tellico-4.2.1/src/gui/tablefieldwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tablefieldwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7Tellico3GUI16TableFieldWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Tellico::GUI::TableFieldWidget::qt_create_metaobjectdata<qt_meta_tag_ZN7Tellico3GUI16TableFieldWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Tellico::GUI::TableFieldWidget",
        "clearImpl",
        "",
        "tableContextMenu",
        "QPoint",
        "point",
        "horizontalHeaderContextMenu",
        "verticalHeaderContextMenu",
        "slotCheckRows",
        "row",
        "col",
        "slotResizeColumn",
        "QTableWidgetItem*",
        "item",
        "slotRenameColumn",
        "slotInsertRow",
        "slotRemoveRow",
        "slotMoveRowUp",
        "slotMoveRowDown",
        "slotClear"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'clearImpl'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tableContextMenu'
        QtMocHelpers::SlotData<void(QPoint)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'horizontalHeaderContextMenu'
        QtMocHelpers::SlotData<void(QPoint)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'verticalHeaderContextMenu'
        QtMocHelpers::SlotData<void(QPoint)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotCheckRows'
        QtMocHelpers::SlotData<void(int, int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'slotResizeColumn'
        QtMocHelpers::SlotData<void(QTableWidgetItem *)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotRenameColumn'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotInsertRow'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRemoveRow'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveRowUp'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveRowDown'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotClear'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TableFieldWidget, qt_meta_tag_ZN7Tellico3GUI16TableFieldWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Tellico::GUI::TableFieldWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<FieldWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico3GUI16TableFieldWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico3GUI16TableFieldWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7Tellico3GUI16TableFieldWidgetE_t>.metaTypes,
    nullptr
} };

void Tellico::GUI::TableFieldWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TableFieldWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clearImpl(); break;
        case 1: _t->tableContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->horizontalHeaderContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 3: _t->verticalHeaderContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 4: _t->slotCheckRows((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->slotResizeColumn((*reinterpret_cast<std::add_pointer_t<QTableWidgetItem*>>(_a[1]))); break;
        case 6: _t->slotRenameColumn(); break;
        case 7: _t->slotInsertRow(); break;
        case 8: _t->slotRemoveRow(); break;
        case 9: _t->slotMoveRowUp(); break;
        case 10: _t->slotMoveRowDown(); break;
        case 11: _t->slotClear(); break;
        default: ;
        }
    }
}

const QMetaObject *Tellico::GUI::TableFieldWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Tellico::GUI::TableFieldWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7Tellico3GUI16TableFieldWidgetE_t>.strings))
        return static_cast<void*>(this);
    return FieldWidget::qt_metacast(_clname);
}

int Tellico::GUI::TableFieldWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FieldWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
